@@ -1,14 +1,14 @@
-'use strict'
+const Arborist = require('@npmcli/arborist')
+const { readFileSync } = require('fs')
+const { join } = require('path')
+const log = require('proc-log')
+const { run, CWD, pkg, fs } = require('./util.js')
 
 // Generates our dependency graph documents in DEPENDENCIES.md.
 
-const Arborist = require('@npmcli/arborist')
-const fs = require('fs')
-
 // To re-create npm-cli-repos.txt run:
-/* eslint-disable-next-line max-len */
-// npx --package=@npmcli/stafftools@latest gh repos --json | json -a name | sort > scripts/npm-cli-repos.txt
-const repos = fs.readFileSync('./scripts/npm-cli-repos.txt', 'utf8').trim().split('\n')
+// npx -p @npmcli/stafftools gh repos --json | json -a name | sort > scripts/npm-cli-repos.txt
+const repos = readFileSync(join(CWD, 'scripts', 'npm-cli-repos.txt'), 'utf-8').trim().split('\n')
 
 // these have a different package name than the repo name, and are ours.
 const aliases = {
@@ -20,7 +20,6 @@ const aliases = {
 // If we see a bare package with just this name, it's NOT ours
 const namespaced = [
   'arborist',
-  'ci-detect',
   'config',
   'disparity-colors',
   'eslint-config',
@@ -66,6 +65,7 @@ function escapeName (name) {
   }
   return name
 }
+
 function stripName (name) {
   if (name.startsWith('@')) {
     const parts = name.slice(1).split('/')
@@ -75,21 +75,19 @@ function stripName (name) {
 }
 
 const main = async function () {
-  const arborist = new Arborist({
-    prefix: process.cwd(),
-    path: process.cwd(),
-  })
-  const tree = await arborist.loadVirtual({ path: process.cwd(), name: 'npm' })
+  // add all of the cli's public workspaces as package names
+  for (const { name, pkg: ws } of await pkg.mapWorkspaces()) {
+    if (!ws.private) {
+      repos.push(name)
+    }
+  }
+
+  const arborist = new Arborist({ prefix: CWD, path: CWD })
+  const tree = await arborist.loadVirtual({ path: CWD, name: 'npm' })
   tree.name = 'npm'
 
-  const {
-    heirarchy: heirarchyOurs,
-    annotations: annotationsOurs,
-  } = walk(tree, true)
-
-  const {
-    annotations: annotationsAll,
-  } = walk(tree, false)
+  const [annotationsOurs, heirarchyOurs] = walk(tree, true)
+  const [annotationsAll] = walk(tree, false)
 
   const out = [
     '# npm dependencies',
@@ -114,39 +112,61 @@ const main = async function () {
     '',
     ` - ${heirarchyOurs.reverse().join('\n - ')}`,
   ]
-  fs.writeFileSync('DEPENDENCIES.md', out.join('\n'))
-  console.log('wrote to DEPENDENCIES.md')
+
+  return fs.writeFile(join(CWD, 'DEPENDENCIES.md'), out.join('\n'))
 }
 
 const walk = function (tree, onlyOurs) {
   const annotations = [] // mermaid dependency annotations
   const dependedBy = {}
+
   iterate(tree, dependedBy, annotations, onlyOurs)
+
   const allDeps = new Set(Object.keys(dependedBy))
   const foundDeps = new Set()
   const heirarchy = []
-  while (allDeps.size) {
-    const level = []
-    for (const dep of allDeps) {
-      if (!dependedBy[dep].size) {
-        level.push(dep)
-        foundDeps.add(dep)
+
+  if (onlyOurs) {
+    while (allDeps.size) {
+      log.silly('SIZE', allDeps.size)
+      const level = []
+
+      for (const dep of allDeps) {
+        log.silly(dep, '::', [...dependedBy[dep]].join(', '))
+        log.silly('-'.repeat(80))
+
+        if (!dependedBy[dep].size) {
+          level.push(dep)
+          foundDeps.add(dep)
+        }
       }
-    }
-    for (const dep of allDeps) {
-      for (const found of foundDeps) {
-        allDeps.delete(found)
-        dependedBy[dep].delete(found)
+
+      log.silly('LEVEL', level.length)
+      log.silly('FOUND', foundDeps.size)
+
+      for (const dep of allDeps) {
+        for (const found of foundDeps) {
+          allDeps.delete(found)
+          dependedBy[dep].delete(found)
+        }
       }
+
+      log.silly('SIZE', allDeps.size)
+
+      if (!level.length) {
+        const remaining = `Remaining deps: ${[...allDeps.keys()]}`
+        throw new Error(`Would do an infinite loop here, need to debug. ${remaining}`)
+      }
+
+      heirarchy.push(level.join(', '))
+      log.silly('HIEARARCHY', heirarchy.length)
+      log.silly('='.repeat(80))
     }
-    if (!level.length) {
-      throw new Error('Would do an infinite loop here, need to debug')
-    }
-    heirarchy.push(level.join(', '))
   }
 
-  return { heirarchy, annotations }
+  return [annotations, heirarchy]
 }
+
 const iterate = function (node, dependedBy, annotations, onlyOurs) {
   if (!dependedBy[node.packageName]) {
     dependedBy[node.packageName] = new Set()
@@ -166,11 +186,4 @@ const iterate = function (node, dependedBy, annotations, onlyOurs) {
   }
 }
 
-main().then(() => {
-  process.exit(0)
-  return 0
-}).catch(err => {
-  console.error(err)
-  process.exit(1)
-  return 1
-})
+run(main)
