@@ -1,5 +1,6 @@
 'use strict'
 
+const nock = require('nock')
 const t = require('tap')
 const Arborist = require('..')
 
@@ -7,8 +8,8 @@ const q = require('../lib/query-selector-all.js')
 
 // test helper that spits out pkgid for readability
 // and deduplicates link/target from results
-const querySelectorAll = async (tree, query) => {
-  const res = await q(tree, query)
+const querySelectorAll = async (tree, query, options) => {
+  const res = await q(tree, query, options)
   return [...new Set(res.map(i => i.pkgid))]
 }
 
@@ -36,6 +37,76 @@ t.test('query-selector-all', async t => {
         └── recur@1.0.0 (recursive production dep of recur, deduped)
 
    */
+
+  const now = Date.now()
+  const today = new Date(now)
+  const yesterday = new Date(now - (1000 * 60 * 60 * 24))
+  const dayBeforeYesterday = new Date(now - (1000 * 60 * 60 * 24 * 2))
+  // @npmcli/abbrev is deliberately left out of this list to cover the case when
+  // fetching a packument fails
+  const packumentStubs = {
+    a: {
+      '1.0.0': today,
+    },
+    abbrev: {
+      '1.1.1': dayBeforeYesterday,
+      '1.2.0': yesterday,
+    },
+    b: {
+      '1.0.0': today,
+    },
+    bar: {
+      '1.4.0': dayBeforeYesterday,
+      '2.0.0': today,
+    },
+    baz: {
+      // undefined for coverage in --before mode
+      '1.0.0': undefined,
+      '1.0.1': yesterday,
+    },
+    'dash-separated-pkg': {
+      '1.0.0': dayBeforeYesterday,
+      '2.0.0': yesterday,
+    },
+    dasher: {
+      '2.0.0': today,
+    },
+    foo: {
+      '2.2.2': today,
+    },
+    lorem: {
+      '1.0.0': today,
+    },
+    moo: {
+      '3.0.0': today,
+    },
+    recur: {
+      '1.0.0': today,
+    },
+    sive: {
+      '1.0.0': today,
+    },
+  }
+
+  nock.disableNetConnect()
+  t.teardown(() => {
+    nock.enableNetConnect()
+  })
+
+  for (const [pkg, versions] of Object.entries(packumentStubs)) {
+    nock('https://registry.npmjs.org')
+      .persist()
+      .get(`/${pkg}`)
+      .reply(200, {
+        time: Object.entries(versions).reduce((final, [version, time]) => {
+          return { ...final, [version]: time }
+        }, {}),
+        versions: Object.keys(versions).reduce((final, next) => {
+          return { ...final, [next]: {} }
+        }, {}),
+      })
+  }
+
   const path = t.testdir({
     node_modules: {
       '@npmcli': {
@@ -52,6 +123,9 @@ t.test('query-selector-all', async t => {
           name: 'abbrev',
           version: '1.1.1',
           license: 'ISC',
+          engines: {
+            node: '^16.0.0',
+          },
         }),
       },
       b: t.fixture('symlink', '../b'),
@@ -62,6 +136,9 @@ t.test('query-selector-all', async t => {
           dependencies: {
             moo: '3.0.0',
           },
+          engines: {
+            node: '>= 14.0.0',
+          },
           arbitrary: {
             foo: [
               false,
@@ -89,6 +166,10 @@ t.test('query-selector-all', async t => {
           scripts: {
             test: 'tap',
           },
+          engines: {
+            // intentionally invalid range
+            node: 'nope',
+          },
         }),
       },
       foo: {
@@ -254,6 +335,18 @@ t.test('query-selector-all', async t => {
     'should throw in invalid selector'
   )
 
+  t.rejects(
+    q(tree, ':semver(1.0.0, [version], eqqq)'),
+    { code: 'EQUERYINVALIDOPERATOR' },
+    'should throw on invalid semver operator'
+  )
+
+  t.rejects(
+    q(tree, ':semver(nope)'),
+    { code: 'EQUERYINVALIDSEMVER' },
+    'should throw on invalid semver value'
+  )
+
   // :scope pseudo
   const [nodeFoo] = await q(tree, '#foo')
   const scopeRes = await querySelectorAll(nodeFoo, ':scope')
@@ -266,13 +359,22 @@ t.test('query-selector-all', async t => {
   ], ':scope > *')
 
   const runSpecParsing = async testCase => {
-    for (const [selector, expected] of testCase) {
-      t.test(selector, async t => {
-        const res = await querySelectorAll(tree, selector)
+    for (const [selector, expected, options = {}] of testCase) {
+      let title = selector
+      if (options.before) {
+        const friendlyTime = options.before === today
+          ? 'today'
+          : options.before === yesterday
+            ? 'yesterday'
+            : options.before
+        title += ` before ${friendlyTime}`
+      }
+      t.test(title, async t => {
+        const res = await querySelectorAll(tree, selector, options)
         t.same(
           res,
           expected,
-          selector
+          title
         )
       })
     }
@@ -407,9 +509,9 @@ t.test('query-selector-all', async t => {
     ]],
 
     // has pseudo
-    [':root > *:has(* > #bar@1.4.0)', ['foo@2.2.2']],
-    ['*:has(* > #bar@1.4.0)', ['foo@2.2.2']],
-    ['*:has(> #bar@1.4.0)', ['foo@2.2.2']],
+    [':root > *:has(* > #bar:semver(1.4.0))', ['foo@2.2.2']],
+    ['*:has(* > #bar:semver(1.4.0))', ['foo@2.2.2']],
+    ['*:has(> #bar:semver(1.4.0))', ['foo@2.2.2']],
     ['.workspace:has(> * > #lorem)', ['a@1.0.0']],
     ['.workspace:has(* #lorem, ~ #b)', ['a@1.0.0']],
 
@@ -559,6 +661,154 @@ t.test('query-selector-all', async t => {
     ]],
     [':semver(=1.4.0)', ['bar@1.4.0']],
     [':semver(1.4.0 || 2.2.2)', ['foo@2.2.2', 'bar@1.4.0']],
+    [':semver(^16.0.0, :attr(engines, [node]))', ['abbrev@1.1.1', 'bar@2.0.0']],
+    [':semver(18.0.0, :attr(engines, [node]))', ['bar@2.0.0']],
+    [':semver(^16.0.0, :attr(engines, [node^=">="]))', ['bar@2.0.0']],
+    [':semver(3.0.0, [version], eq)', ['moo@3.0.0']],
+    [':semver(^3.0.0, [version], eq)', []],
+    [':semver(1.0.0, [version], neq)', [
+      '@npmcli/abbrev@2.0.0-beta.45',
+      'abbrev@1.1.1',
+      'bar@2.0.0',
+      'dasher@2.0.0',
+      'foo@2.2.2',
+      'bar@1.4.0',
+      'moo@3.0.0',
+    ]],
+    [':semver(^1.0.0, [version], neq)', []],
+    [':semver(2.0.0, [version], gt)', ['foo@2.2.2', 'moo@3.0.0']],
+    [':semver(^2.0.0, [version], gt)', []],
+    [':semver(2.0.0, [version], gte)', [
+      'bar@2.0.0',
+      'dasher@2.0.0',
+      'foo@2.2.2',
+      'moo@3.0.0',
+    ]],
+    [':semver(^2.0.0, [version], gte)', []],
+    [':semver(1.1.1, [version], lt)', [
+      'query-selector-all-tests@1.0.0',
+      'a@1.0.0',
+      'b@1.0.0',
+      'baz@1.0.0',
+      'dash-separated-pkg@1.0.0',
+      'ipsum@npm:sit@1.0.0',
+      'lorem@1.0.0',
+      'recur@1.0.0',
+      'sive@1.0.0',
+    ]],
+    [':semver(^1.1.1, [version], lt)', []],
+    [':semver(1.1.1, [version], lte)', [
+      'query-selector-all-tests@1.0.0',
+      'a@1.0.0',
+      'b@1.0.0',
+      'abbrev@1.1.1',
+      'baz@1.0.0',
+      'dash-separated-pkg@1.0.0',
+      'ipsum@npm:sit@1.0.0',
+      'lorem@1.0.0',
+      'recur@1.0.0',
+      'sive@1.0.0',
+    ]],
+    [':semver(^1.1.1, [version], lte)', []],
+    [':semver(^14.0.0, :attr(engines, [node]), intersects)', ['bar@2.0.0']],
+    [':semver(>=14, :attr(engines, [node]), subset)', ['abbrev@1.1.1', 'bar@2.0.0']],
+    [':semver(^2.0.0, [version], gtr)', ['moo@3.0.0']],
+    [':semver(^2.0.0, :attr(engines, [node]), gtr)', []],
+    [':semver(20.0.0, :attr(engines, [node]), gtr)', ['abbrev@1.1.1']],
+    [':semver(1.0.1, [version], gtr)', [
+      'query-selector-all-tests@1.0.0',
+      'a@1.0.0',
+      'b@1.0.0',
+      'baz@1.0.0',
+      'dash-separated-pkg@1.0.0',
+      'ipsum@npm:sit@1.0.0',
+      'lorem@1.0.0',
+      'recur@1.0.0',
+      'sive@1.0.0',
+    ]],
+    [':semver(^1.1.1, [version], ltr)', [
+      'query-selector-all-tests@1.0.0',
+      'a@1.0.0',
+      'b@1.0.0',
+      'baz@1.0.0',
+      'dash-separated-pkg@1.0.0',
+      'ipsum@npm:sit@1.0.0',
+      'lorem@1.0.0',
+      'recur@1.0.0',
+      'sive@1.0.0',
+    ]],
+    [':semver(^1.1.1, :attr(engines, [node]), ltr)', []],
+    [':semver(0.0.1, :attr(engines, [node]), ltr)', ['abbrev@1.1.1', 'bar@2.0.0']],
+    [':semver(1.1.1, [version], ltr)', [
+      '@npmcli/abbrev@2.0.0-beta.45',
+      'bar@2.0.0',
+      'dasher@2.0.0',
+      'foo@2.2.2',
+      'bar@1.4.0',
+      'moo@3.0.0',
+    ]],
+
+    // outdated pseudo
+    [':outdated', [
+      'abbrev@1.1.1', // 1.2.0 is available
+      'baz@1.0.0', // 1.0.1 is available
+      'dash-separated-pkg@1.0.0', // 2.0.0 is available
+      'bar@1.4.0', // 2.0.0 is available
+    ]],
+    [':outdated(any)', [
+      'abbrev@1.1.1', // 1.2.0 is available
+      'baz@1.0.0', // 1.0.1 is available
+      'dash-separated-pkg@1.0.0', // 2.0.0 is available
+      'bar@1.4.0', // 2.0.0 is available
+    ]],
+    [':outdated(major)', [
+      'dash-separated-pkg@1.0.0', // 2.0.0 is available
+      'bar@1.4.0', // 2.0.0 is available
+    ]],
+    [':outdated(minor)', [
+      'abbrev@1.1.1', // 1.2.0 is available
+    ]],
+    [':outdated(patch)', [
+      'baz@1.0.0', // 1.0.1 is available
+    ]],
+    [':outdated(in-range)', [
+      'abbrev@1.1.1', // 1.2.0 is available and in-range
+      'baz@1.0.0', // 1.0.1 is available and in-range
+    ]],
+    [':outdated(out-of-range)', [
+      'dash-separated-pkg@1.0.0', // 2.0.0 is available
+      'bar@1.4.0', // 2.0.0 is available and out-of-range
+    ]],
+    [':outdated(nonsense)', []], // invalid, no results ever
+
+    // :outdated combined with --before
+    [':outdated', [
+      'abbrev@1.1.1', // 1.2.0 is available and published yesterday
+      'baz@1.0.0', // 1.0.1 is available and published yesterday
+      'dash-separated-pkg@1.0.0', // 2.0.0 is available and published yesterday
+    ], { before: yesterday }],
+    [':outdated(any)', [
+      'abbrev@1.1.1', // 1.2.0 is available and published yesterday
+      'baz@1.0.0', // 1.0.1 is available and published yesterday
+      'dash-separated-pkg@1.0.0', // 2.0.0 is available and published yesterday
+    ], { before: yesterday }],
+    [':outdated(major)', [
+      'dash-separated-pkg@1.0.0', // 2.0.0 is available and published yesterday
+    ], { before: yesterday }],
+    [':outdated(minor)', [
+      'abbrev@1.1.1', // 1.2.0 is available and published yesterday
+    ], { before: yesterday }],
+    [':outdated(patch)', [
+      'baz@1.0.0', // 1.0.1 is available and published yesterday
+    ], { before: yesterday }],
+    [':outdated(in-range)', [
+      'abbrev@1.1.1', // 1.2.0 is available, in-range and published yesterday
+      'baz@1.0.0', // 1.0.1 is available, in-range and published yesterday
+    ], { before: yesterday }],
+    [':outdated(out-of-range)', [
+      'dash-separated-pkg@1.0.0', // 2.0.0 is available, out-of-range and published yesterday
+    ], { before: yesterday }],
+    [':outdated(nonsense)', [], { before: yesterday }], // again, no results here ever
 
     // attr pseudo
     [':attr([name=dasher])', ['dasher@2.0.0']],
@@ -681,8 +931,8 @@ t.test('query-selector-all', async t => {
 
     // id selector
     ['#bar', ['bar@2.0.0', 'bar@1.4.0']],
-    ['#bar@2.0.0', ['bar@2.0.0']],
-    ['#@npmcli/abbrev@2.0.0-beta.45', ['@npmcli/abbrev@2.0.0-beta.45']],
+    ['#bar:semver(2.0.0)', ['bar@2.0.0']],
+    ['#@npmcli/abbrev:semver(2.0.0-beta.45)', ['@npmcli/abbrev@2.0.0-beta.45']],
     ['#bar:semver(2.0)', ['bar@2.0.0']],
     ['#bar:semver(2)', ['bar@2.0.0']],
     ['#bar:semver(^2.0.0)', ['bar@2.0.0']],
@@ -709,6 +959,6 @@ t.test('query-selector-all', async t => {
     [':root #bar:semver(1) > *', ['dasher@2.0.0']],
     [':root #bar:semver(1) ~ *', ['dash-separated-pkg@1.0.0']],
     ['#bar:semver(2), #foo', ['bar@2.0.0', 'foo@2.2.2']],
-    ['#a, #bar:semver(2), #foo@2.2.2', ['a@1.0.0', 'bar@2.0.0', 'foo@2.2.2']],
+    ['#a, #bar:semver(2), #foo:semver(2.2.2)', ['a@1.0.0', 'bar@2.0.0', 'foo@2.2.2']],
   ])
 })
